@@ -45,35 +45,55 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     29 => -1,
                     31 => -1,
                     32 => -2,
+                    33 => -2,
+                    35 => 1,
                 ],
             ]),
             RawCodeCoverageData::fromXdebugWithoutPathCoverage([
                 TEST_FILES_PATH . 'BankAccount.php' => [
                     8  => 1,
+                    9  => -2,
                     13 => 1,
+                    14 => -1,
+                    15 => -1,
                     16 => 1,
+                    18 => -1,
                     29 => 1,
+                    31 => -1,
+                    32 => -2,
+                    33 => -2,
                 ],
             ]),
             RawCodeCoverageData::fromXdebugWithoutPathCoverage([
                 TEST_FILES_PATH . 'BankAccount.php' => [
                     8  => 1,
+                    9  => -2,
                     13 => 1,
+                    14 => -1,
+                    15 => -1,
                     16 => 1,
+                    18 => -1,
                     22 => 1,
+                    24 => -1,
+                    25 => -2,
                 ],
             ]),
             RawCodeCoverageData::fromXdebugWithoutPathCoverage([
                 TEST_FILES_PATH . 'BankAccount.php' => [
                     8  => 1,
+                    9  => -2,
                     13 => 1,
                     14 => 1,
                     15 => 1,
+                    16 => -1,
                     18 => 1,
                     22 => 1,
                     24 => 1,
+                    25 => -2,
                     29 => 1,
                     31 => 1,
+                    32 => -2,
+                    33 => -2,
                 ],
             ]),
         ];
@@ -276,9 +296,15 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                 TEST_FILES_PATH . 'BankAccount.php' => [
                     'lines' => [
                         8  => 1,
+                        9  => -2,
                         13 => 1,
+                        14 => -1,
+                        15 => -1,
                         16 => 1,
+                        18 => -1,
                         29 => 1,
+                        31 => -1,
+                        32 => -2,
                     ],
                     'functions' => [
                         'BankAccount->depositMoney' => [
@@ -457,9 +483,15 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                 TEST_FILES_PATH . 'BankAccount.php' => [
                     'lines' => [
                         8  => 1,
+                        9  => -2,
                         13 => 1,
+                        14 => -1,
+                        15 => -1,
                         16 => 1,
+                        18 => -1,
                         22 => 1,
+                        24 => -1,
+                        25 => -2,
                     ],
                     'functions' => [
                         'BankAccount->depositMoney' => [
@@ -638,14 +670,18 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                 TEST_FILES_PATH . 'BankAccount.php' => [
                     'lines' => [
                         8  => 1,
+                        9  => -2,
                         13 => 1,
                         14 => 1,
                         15 => 1,
+                        16 => -1,
                         18 => 1,
                         22 => 1,
                         24 => 1,
+                        25 => -2,
                         29 => 1,
                         31 => 1,
+                        32 => -2,
                     ],
                     'functions' => [
                         'BankAccount->depositMoney' => [
@@ -1314,12 +1350,10 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     0 => 'BankAccountTest::testBalanceIsInitiallyZero',
                     1 => 'BankAccountTest::testDepositWithdrawMoney',
                 ],
-                9  => null,
                 13 => [],
                 14 => [],
                 15 => [],
                 16 => [],
-                18 => [],
                 22 => [
                     0 => 'BankAccountTest::testBalanceCannotBecomeNegative2',
                     1 => 'BankAccountTest::testDepositWithdrawMoney',
@@ -1327,7 +1361,6 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                 24 => [
                     0 => 'BankAccountTest::testDepositWithdrawMoney',
                 ],
-                25 => null,
                 29 => [
                     0 => 'BankAccountTest::testBalanceCannotBecomeNegative',
                     1 => 'BankAccountTest::testDepositWithdrawMoney',
@@ -1348,12 +1381,10 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                     0 => 'BankAccountTest::testDepositWithdrawMoney',
                     1 => 'BankAccountTest::testBalanceIsInitiallyZero',
                 ],
-                9  => null,
                 13 => [],
                 14 => [],
                 15 => [],
                 16 => [],
-                18 => [],
                 22 => [
                     0 => 'BankAccountTest::testBalanceCannotBecomeNegative2',
                     1 => 'BankAccountTest::testDepositWithdrawMoney',
@@ -1361,7 +1392,6 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
                 24 => [
                     0 => 'BankAccountTest::testDepositWithdrawMoney',
                 ],
-                25 => null,
                 29 => [
                     0 => 'BankAccountTest::testDepositWithdrawMoney',
                     1 => 'BankAccountTest::testBalanceCannotBecomeNegative',
@@ -1688,6 +1718,42 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
         return $stub;
     }
 
+    protected function getLineCoverageForFileWithEval(): CodeCoverage
+    {
+        $filter = new Filter;
+        $filter->includeFile(TEST_FILES_PATH . 'source_with_eval.php');
+
+        $coverage = new CodeCoverage(
+            $this->setUpXdebugStubForFileWithEval(),
+            $filter
+        );
+
+        $coverage->start('FileWithEval', true);
+        $coverage->stop();
+
+        return $coverage;
+    }
+
+    protected function setUpXdebugStubForFileWithEval(): Driver
+    {
+        $stub = $this->createStub(Driver::class);
+
+        $stub->method('stop')
+            ->willReturn(RawCodeCoverageData::fromXdebugWithoutPathCoverage(
+                [
+                    TEST_FILES_PATH . 'source_with_eval.php' => [
+                        3 => 1,
+                        5 => 1,
+                    ],
+                    TEST_FILES_PATH . 'source_with_eval.php(5) : eval()\'d code' => [
+                        1 => 1,
+                    ],
+                ]
+            ));
+
+        return $stub;
+    }
+
     protected function getCoverageForClassWithAnonymousFunction(): CodeCoverage
     {
         $filter = new Filter;
@@ -1776,4 +1842,128 @@ abstract class TestCase extends \PHPUnit\Framework\TestCase
             $fileInfo->isDir() ? rmdir($pathname) : unlink($pathname);
         }
     }
+
+    protected function getCoverageForFilesWithUncoveredIncluded(): CodeCoverage
+    {
+        $data = $this->getLineCoverageXdebugDataForBankAccount();
+
+        $stub = $this->createStub(Driver::class);
+
+        $stub->method('stop')
+            ->will($this->onConsecutiveCalls(...$data));
+
+        $filter = new Filter;
+        $filter->includeFile(TEST_FILES_PATH . 'BankAccount.php');
+        $filter->includeFile(TEST_FILES_PATH . 'NamespacedBankAccount.php');
+
+        $coverage = new CodeCoverage($stub, $filter);
+        $coverage->includeUncoveredFiles();
+
+        $coverage->start(
+            new BankAccountTest('testBalanceIsInitiallyZero'),
+            true
+        );
+
+        $coverage->stop(
+            true,
+            [TEST_FILES_PATH . 'BankAccount.php' => range(6, 9)]
+        );
+
+        $coverage->start(
+            new BankAccountTest('testBalanceCannotBecomeNegative')
+        );
+
+        $coverage->stop(
+            true,
+            [TEST_FILES_PATH . 'BankAccount.php' => range(27, 32)]
+        );
+
+        $coverage->start(
+            new BankAccountTest('testBalanceCannotBecomeNegative2')
+        );
+
+        $coverage->stop(
+            true,
+            [TEST_FILES_PATH . 'BankAccount.php' => range(20, 25)]
+        );
+
+        $coverage->start(
+            new BankAccountTest('testDepositWithdrawMoney')
+        );
+
+        $coverage->stop(
+            true,
+            [
+                TEST_FILES_PATH . 'BankAccount.php' => array_merge(
+                    range(6, 9),
+                    range(20, 25),
+                    range(27, 32)
+                ),
+            ]
+        );
+
+        return $coverage;
+    }
+
+    protected function getCoverageForFilesWithUncoveredExcluded(): CodeCoverage
+    {
+        $data = $this->getLineCoverageXdebugDataForBankAccount();
+
+        $stub = $this->createStub(Driver::class);
+
+        $stub->method('stop')
+            ->will($this->onConsecutiveCalls(...$data));
+
+        $filter = new Filter;
+        $filter->includeFile(TEST_FILES_PATH . 'BankAccount.php');
+        $filter->includeFile(TEST_FILES_PATH . 'NamespacedBankAccount.php');
+
+        $coverage = new CodeCoverage($stub, $filter);
+        $coverage->excludeUncoveredFiles();
+
+        $coverage->start(
+            new BankAccountTest('testBalanceIsInitiallyZero'),
+            true
+        );
+
+        $coverage->stop(
+            true,
+            [TEST_FILES_PATH . 'BankAccount.php' => range(6, 9)]
+        );
+
+        $coverage->start(
+            new BankAccountTest('testBalanceCannotBecomeNegative')
+        );
+
+        $coverage->stop(
+            true,
+            [TEST_FILES_PATH . 'BankAccount.php' => range(27, 32)]
+        );
+
+        $coverage->start(
+            new BankAccountTest('testBalanceCannotBecomeNegative2')
+        );
+
+        $coverage->stop(
+            true,
+            [TEST_FILES_PATH . 'BankAccount.php' => range(20, 25)]
+        );
+
+        $coverage->start(
+            new BankAccountTest('testDepositWithdrawMoney')
+        );
+
+        $coverage->stop(
+            true,
+            [
+                TEST_FILES_PATH . 'BankAccount.php' => array_merge(
+                    range(6, 9),
+                    range(20, 25),
+                    range(27, 32)
+                ),
+            ]
+        );
+
+        return $coverage;
+    }
 }
